structure T = Tokens
structure I = Interface
structure A = Absyn

open I

type pos = I.pos
type svalue = T.svalue
type ('a,'b) token = ('a,'b) T.token
type lexresult = (svalue,pos) token list * string

fun eof() = ([T.EOF(!line,!line)],"")

fun CNs s t       = ([T.CNs(A.ConS s,!line,!line)],t)
fun CNp s t       = ([T.CNp(A.ConS s,!line,!line)],t)
fun RNs s t       = ([T.RNs(A.ConR s,!line,!line)],t)
fun RNp s t       = ([T.RNp(A.ConR s,!line,!line)],t)
fun PN(s:string)  = ([T.PN(A.ConS s,!line,!line)],s)
fun IVs s t       = ([T.IVs(A.ConS s,!line,!line)],t)
fun IVp(s:string) = ([T.IVp(A.ConS s,!line,!line)],s)
fun TVs s t       = ([T.TVs(A.ConR s,!line,!line)],t)
fun TVp(s:string) = ([T.TVp(A.ConR s,!line,!line)],s)
fun CA(s:string)  = ([T.CA(A.ConS s,!line,!line)],s)
fun CAs s t       = ([T.CAs(A.ConS s,!line,!line)],t)
fun CAp s t       = ([T.CAp(A.ConS s,!line,!line)],t)
fun P(s:string)   = ([T.P(A.ConR s,!line,!line)],s)
fun IA s t        = ([T.IA(A.ConR s,!line,!line)],t)
fun IAcomp s t    = ([T.IAcomp(A.Tc(A.ConR s),!line,!line)],t)

%%  
%header (functor NL_LexFun(structure Absyn: ABSYN
                           structure Tokens: NL__TOKENS
                              sharing type Absyn.set = Tokens.set 
                           structure Interface: INTERFACE) : LEXER);
%full 

whitespace=[\t\ ]*;
%%
<INITIAL>{whitespace} => (lex());
\n         => (next_line(); lex());
"."        => ([T.PUNKT(!line,!line)],yytext);
"ist"      => ([T.CopVs(!line,!line)],yytext);
"sind"     => ([T.CopVp(!line,!line)],yytext);
"von"    => ([T.von(!line,!line)],yytext);
"als"    => ([T.als(!line,!line)],yytext);
"und"    => ([T.und(!line,!line)],yytext);
"oder"   => ([T.oder(!line,!line)],yytext);
("der" | "die" | "das" | "den")   
           => ([T.DET(!line,!line)],yytext);
("Ein" | "Eine" |"einen" | "ein" | "eine" | "eines")   
           => ([T.EQ(!line,!line)],yytext);
("Kein" | "Keine" | "keinen" | "kein" | "keine" | "keines")   
           => ([T.NQ(!line,!line)],yytext);
("Alle" | "alle" | "Jeder" | "jeder" 
                 | "Jede" | "jede" | "Jeden" | "jeden" | "jedes")   
           => ([T.UQ(!line,!line)],yytext);
"Anna"     => (PN "Anna");
"Dieter"   => (PN "Dieter");
"Emil"     => (PN "Emil");
"Maria"    => (PN "Maria");
"Waldi"    => (PN "Waldi");
"Rom"      => (PN "Rom");
"Madrid"   => (PN "Madrid");
"Italien"  => (PN "Italien");
"Spanien"  => (PN "Spanien");
"Mann"     => (CNs "Mann" yytext);
"Frau"     => (CNs "Frau" yytext);
"Männer"   => (CNp "Mann" yytext);
"Frauen"   => (CNp "Frau" yytext);
"Hund"     => (CNs "Hund" yytext);
"Hunde"    => (CNp "Hund" yytext);           
"Berg"     => (CNs "Berg" yytext);
"Berge"    => (CNp "Berg" yytext);           
"Student"  => (CNs "Student" yytext);
"Studenten"  => (CNp "Student" yytext);           
"Nachbar"  => (RNs "Nachbar" yytext);
"Nachbarn" => (RNp "Nachbar" yytext);             
"Bruder"   => (RNs "Bruder" yytext);
"Brüder"   => (RNp "Bruder" yytext);              
"Schwester"  => (RNs "Schwester" yytext);
"Schwestern" => (RNp "Schwester" yytext);
"Hauptstadt"   => (RNs "Hauptstadt" yytext);
"Hauptstädte"  => (RNp "Hauptstadt" yytext);
"Stadt"    => (CNs "Stadt" yytext);
"Städte"   => (CNp "Stadt" yytext);
"gesund"   => (CA "gesund");
"braun"    => (CA "braun");
"braune"    => (CAp "braun" yytext);
"brauner" | "braunes"
            => (CAs "braun" yytext);
"toll"      => (CA "toll");
"tolle"     => (CAp "toll" yytext);
("toller" | "tolles")
           => (CAs "toll" yytext);
"nett"     => (CA "nett");
"nette"    => (CAp "nett" yytext);
("netter" | "nettes")
           => (CAs "nett" yytext);
"weiß"     => (CA "weiß");
"weiße"    => ([T.CAp(A.ConS "weiß",!line,!line),T.CAs(A.ConS "weiß",!line,!line)],yytext);
"weißer" | "weißes"
            => (CAs "weiß" yytext);
"in"       => (P "in");
"arbeiten" => (IVp "arbeiten");
"arbeitet" => (IVs "arbeiten" yytext);
"kennen"   => (TVp "kennen");
"kennt"    => (TVs "kennen" yytext);
"lieben"   => (TVp "lieben");
"liebt"    => (TVs "lieben" yytext);
"schöner"   => (IAcomp "schöner" yytext);
"älter"     => (IAcomp "älter" yytext);
"größer"   => (IAcomp "größer" yytext);
"kleiner"  => (IAcomp "kleiner" yytext);
("alt"|"alter"|"alte"|"altes") => (IA "älter" yytext);
("groß"|"großer"|"große"|"großes") 
                               => (IA "größer" yytext);
("klein"|"kleiner"|"kleine"|"kleines") 
                               => (IA "kleiner" yytext);
"Ameise"      => (CNs "Ameise" yytext);
"Ameisen"     => (CNp "Ameise" yytext);                               
"Elephant"    => (CNs "Elephant" yytext);
"Elephanten"  => (CNp "Elephant" yytext);                               
"Tier"        => (CNs "Tier" yytext);
"Tiere"       => (CNp "Tier" yytext);                               
.          => (error ("ignoring illegal character " ^ yytext,
                      !line,!line); lex());


