structure T = Tokens
structure I = Interface
structure A = Absyn

open I

type pos = I.pos
type svalue = T.svalue
type ('a,'b) token = ('a,'b) T.token
type lexresult = (svalue,pos) token list * string

fun eof() = ([T.EOF(!line,!line)],"")

fun CN s t       = ([T.CN(A.ConS s,!line,!line)],t)
fun RN s t       = ([T.RN(A.ConR s,!line,!line)],t)
fun PN(s:string) = ([T.PN(A.ConS s,!line,!line)],s)
fun IV s t       = ([T.IV(A.ConS s,!line,!line)],t)
fun TV(s:string) = ([T.TV(A.ConR s,!line,!line)],s)
fun CA(s:string) = ([T.CA(A.ConS s,!line,!line)],s)
fun P(s:string) = ([T.P(A.ConR s,!line,!line)],s)
fun IA s t       = ([T.IA(A.ConR s,!line,!line)],t)
fun IAcomp s t   = ([T.IAcomp(A.Tc(A.ConR s),!line,!line)],t)

%%  
%header (functor NL_LexFun(structure Absyn: ABSYN
                           structure Tokens: NL__TOKENS
                              sharing type Absyn.set = Tokens.set 
                           structure Interface: INTERFACE) : LEXER);
%full 

whitespace=[\t\ ]*;
%%
<INITIAL>{whitespace} => (lex());
\n         => (next_line(); lex());
"."        => ([T.PUNKT(!line,!line)],yytext);
"ist"      => ([T.CopV(!line,!line)],yytext);
"von"    => ([T.von(!line,!line)],yytext);
"als"    => ([T.als(!line,!line)],yytext);
"und"    => ([T.und(!line,!line)],yytext);
"oder"   => ([T.oder(!line,!line)],yytext);
("der" | "die" | "das" | "den")   
           => ([T.Det(!line,!line)],yytext);
("Ein" | "Eine" |"einen" | "ein" | "eine" | "eines")   
           => ([T.EQ(!line,!line)],yytext);
("Kein" | "Keine" | "keinen" | "kein" | "keine" | "keines")   
           => ([T.NQ(!line,!line)],yytext);
("Alle" | "alle" | "Jeder" | "jeder" 
                 | "Jede" | "jede" | "Jeden" | "jeden")   
           => ([T.UQ(!line,!line)],yytext);
"Anna"     => (PN "Anna");
"Dieter"   => (PN "Dieter");
"Emil"     => (PN "Emil");
"Maria"    => (PN "Maria");
"Waldi"    => (PN "Waldi");
"Rom"   => (PN "Rom");
"Madrid"   => (PN "Madrid");
"Italien"   => (PN "Italien");
"Spanien"   => (PN "Spanien");
"Mann"     => (CN "Mann" yytext);
"Frau"     => (CN "Frau" yytext);
("Hund" | "Hunde")
           => (CN "Hund" yytext);
"Berg"     => (CN "Berg" yytext);
("Student" | "Studenten")
           => (CN "Student" yytext);
"Nachbar"  => (RN "Nachbar" yytext);  
"Bruder"   => (RN "Bruder" yytext);   
"Schwester"  => (RN "Schwester" yytext); 
"Hauptstadt"   => (RN "Hauptstadt" yytext);
"Stadt"    => (CN "Stadt" yytext);
"gesund"   => (CA "gesund");
("weißer" | "weiße" | "weißes")  => (CA "weiß");           
("braun" | "braune" | "brauner")  => (CA "braun");
("toll" | "tolle" | "toller")     => (CA "toll");
("nett" | "nette" | "netter" | "netten")
          => (CA "nett");
"schön"   => (IA "schön" yytext);
"schöner" => (IAcomp "schöner" yytext);
"älter"   => (IAcomp "älter" yytext);
("in") => (P "in");
("arbeiten" | "arbeitet") => (IV "arbeiten" yytext);
("kennen" | "kennt")      => (TV "kennen");
("lieben" | "liebt")      => (TV "lieben");
"schöner"  => (IAcomp "schöner" yytext);
"älter"    => (IAcomp "älter" yytext);
"größer"   => (IAcomp "größer" yytext);
"kleiner"  => (IAcomp "kleiner" yytext);
("alt"|"alter"|"alte"|"altes") => (IA "älter" yytext);
("groß"|"großer"|"große"|"großes") 
                               => (IA "größer" yytext);
("klein"|"kleiner"|"kleine"|"kleines") 
                               => (IA "kleiner" yytext);
"Ameise"|"Ameisen"      => (CN "Ameise" yytext);
"Hund"|"Hunde"          => (CN "Hund" yytext);
"Elephant"|"Elephanten" => (CN "Elephant" yytext);
"Tier"|"Tiere"          => (CN "Tier" yytext);
.          => (error ("ignoring illegal character " ^ yytext,
                      !line,!line); lex());


